******* 
*
*  makeit.prg
*
*****

parameters raw_file

if pcount() <> 1 
clear
text
Program: MAKEIT.PRG             Extensively modified from FASTCLIP.PRG by
 Author: Kevin Talbot                  Author: David P. Mezler
 Source: AAP476                        Source: BFJ279
 CompuS: 75706,316                     CompuS: 71340,3310
   Date: 10/22/88                       Genie: XTH61688
                                         Date: 3/20/1988
 Description:
   This program completely automates the tedious (and error prone) creation
   of the LNK and MAK files for use with the Clipper compiler and the companion
   linker and make programs. It uses the redirected output from the Clipper
   compiler to sniff out all the module names in your application.

   To properly run this program, run the AUTOMAKE.BAT batch file by typing
   'automake <file>' where <file> is the name of the "top" PRG file, i.e. 
   the name you want for the EXE file.

   The output from MAKEIT.EXE consists of three files:
      <file>.MAK   The "make" file for use my Nantucket's MAKE.EXE
      <file>.LNK   The "link" file for use by the liner (Tlink here)
      -.BAT        A batch file that will call the make utility (I get so
                   tired of typing "make myfile.mak". I like typing just "-".)
endtext
wait
clear
text
 Note: 
   If you are not using the Tlink linker from Borland's Turbo "C",
   edit this file to create a .lnk file compatable to the linker you
   are using.

   This set of programs expects the linker and Clipper are available
   through a dos path.  This file expects S87 or A86 versions clipper.

   If you have set the environment variable "lib" to something, MAKEIT
   will use it. I suggest setting it to "C:\CLIPPER" or whatever directory
   and drive you keep the Clipper libraries in.   

              ******* Warning  *********
   Make sure the file names created by this program do not trash files
   of the same names. I am not responsible for disasters.  Use at your own
   risk.
endtext
wait
quit
endif

raw_file  = upper(trim(raw_file))    && <FILE>  w/o extension
prg_file  = raw_file + '.PRG'        && <FILE>.PRG
make_file = raw_file + '.MAK'        && <FILE>.MAK
link_file = raw_file + '.LNK'        && <FILE>.LNK

if .not. file('&prg_file')
  clear
  @ 10,10 say 'File ' + prg_file + '.PRG does not exist'
  quit
endif


if .not. file('flist.txt')
  clear
  @ 10,10 say 'File Flist.txt does not exist'
  quit
endif

* get the Clipper environment variable, if present
lib_var = gete("LIB")

if empty(lib_var)
 lib_var = "\clipper"
endif

if .not. file('f_work.dbf')
  ? 'Creating f_work.dbf file...'
  ?
  create new_stru
  append blank
  replace field_name with 'file'
  replace field_type with 'C'
  replace field_len with 025
  replace field_dec with 000
  create f_work from new_stru
  erase new_stru.dbf
else
  use f_work
  zap
endif

use f_work
?
? 'Reading flist.txt file to get OBJ file list...'
append from flist.txt sdf

** get rid of extraneous Clipper messages in the file
?
? 'Cleaning up file to extract just the file names...'
delete all for left(file,9) <> 'Compiling'
pack

** trim off leading part of line before the file name
replace all file with substr(file,11,25)

** trim off the file extensions
replace all file with left(file,at('.',file)-1)

* let's get them into an array to speed things up and simplify loops...
num_files = lastrec()
declare fname[num_files]
go top
for i = 1 to 999 
  fname[i] = trim(file)
  skip
  if eof()
    exit
  endif 
next 

** done with the files
delete file flist.txt
delete file f_work.dbf

** create the lnk file....
?
? 'Creating "lnk" file....'
set alternate to &link_file
set alternate on
for i = 1 to num_files
  ?? fname[i] + ' '
next

?? ', ' + raw_file                               && exe file name
?? ',, '                                         && no name for mapfile
?? lib_var + '\clipper ' + lib_var + '\extend'   && library names 

CLOSE alternate                                    


********* create the make file 

?
? 'Creating "MAK" file....'
set alternate to &make_file
set alternate on

? '#  ' + make_file 
? '#  This file created by the "MAKEIT" utility program on ' + dtoc(date())

** write shorthand for dependancy action
?  '.prg.obj:'                 
?  '  clipper $* -m'
?
?  '#  Macros....'

** write macro(s) for easier inclusion in EXE dependancy list

** write 'per_line' obj file names per macro line  
per_line = 5
for line = 0 to 999
  ? 'files' + ltrim(str(line,3)) + '='
  for i = 1 to per_line
    indx = i + (line * per_line) 
    ?? fname[indx] + '.OBJ '
    if indx >= num_files
      exit
    endif
  next
  if indx >= num_files
    num_macros = line
    exit
  endif
next


*** next, the list of OBJ and PRG dependencies.....

?
? '# OBJ/PRG dependencies....'
go top

for i = 1 to num_files
  ? fname[i] + '.OBJ' + space(15 - len(fname[i])) + ':' + fname[i] + '.PRG'
next 


*** finally, write the EXE dependancy rule....
?
? '# EXE dependencies ....'
? raw_file + '.EXE: '

for i = 0 to num_macros
  ?? '$(files' + ltrim(str(i,2)) + ') '
next
** /x means 'no map files' to tlink
? ' tlink /x @' + link_file
?
close alternate

?
? 'Creating "-.BAT" file....'
set alternate to -.bat
? 'echo off'
? 'echo "-.bat" now executing to make ' + raw_file + '.EXE ....'
? 'make ' + raw_file + '.mak'
close alternate

?
? 'All done! You should now just be able to type "-" to invoke the "-.BAT"'
? 'file to call the make utility and rebuild your application.'
?
?

quit
*=-=-=-=-=-=-=-=-=-=-=-=- eof makeit.prg -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

